@@ -28,41 +28,22 @@ module G = Gui_global
 module Mi = Gui_misc
 
 (*module Gui_rooms = Gui_rooms2*)
-let chmod_config () =
-	let base_config = 
-	    (Filename.concat CommonOptions.home_basedir ".mldonkey_gui.ini")
-	in
-	let save_config =
-		base_config^".old"
-	in
-	begin
-	if Sys.file_exists base_config then
-		Unix.chmod base_config 0o600
-	else
-		()
-	end;
-	begin
-	if Sys.file_exists save_config then
-		Unix.chmod save_config 0o600
-	else
-		()
-	end
   
 let _ = 
   (try Options.load O.mldonkey_gui_ini with
       Sys_error _ ->
 	(try Options.save O.mldonkey_gui_ini with _ -> ())
      | e ->
-        lprintf "Exception %s in load options %s" 
+        lprintf "Exception %s in load options %s\n" 
 	 (Printexc2.to_string e)
 	 (Options.options_file_name O.mldonkey_gui_ini);
-        lprint_newline ();
+        
   );
   let args = 
     ("-dump_msg", Arg.Unit (fun _ ->
           Options.save Gui_messages.message_file
       ), ": update internationalisation message file")::
-    Options.simple_args O.mldonkey_gui_ini in
+    Options.simple_args "" O.mldonkey_gui_ini in
   Arg.parse args (Arg.usage args)  "mlgui: the GUI to use with mldonkey"
   
 (* Check bindings *)
@@ -158,10 +139,9 @@ let canon_client gui c =
       
       begin
         if c.client_type <> cc.client_type then begin
-            match c.client_type, cc.client_type with
-            | NormalClient, _ ->
+            if c.client_type = 0 then
                 box_friends#h_remove_friend c.client_num
-            | _  ->
+            else
                 box_friends#h_update_friend cc
           end
       end;
@@ -173,9 +153,8 @@ let canon_client gui c =
       cc.client_kind <- c.client_kind;
       cc.client_tags <- c.client_tags;
       begin
-        match c.client_type with
-          NormalClient -> ()
-        | _ -> box_friends#h_update_friend cc;
+        if c.client_type <> 0 then
+          box_friends#h_update_friend cc;
       end;
       
       cc
@@ -183,9 +162,8 @@ let canon_client gui c =
 (*        lprintf "Adding client %d" c.client_num; lprint_newline ();  *)
         Hashtbl.add G.locations c.client_num c;
         begin
-          match c.client_type with
-            NormalClient -> ()
-          | _ -> box_friends#h_update_friend c;
+          if c.client_type <> 0 then
+            box_friends#h_update_friend c;
         end;
         c
   in
@@ -200,7 +178,7 @@ let value_reader gui t =
         lprintf "MESSAGE RECEIVED: %s" 
           (string_of_to_gui t);
         lprint_newline ();
-        
+      
       end;
     
     
@@ -255,24 +233,28 @@ let value_reader gui t =
             s.nshared_files 
             (Gui_misc.size_of_int64 s.upload_counter)
           (s.tcp_upload_rate + s.udp_upload_rate) s.udp_upload_rate
-          (s.tcp_download_rate + s.udp_download_rate) s.udp_download_rate
+            (s.tcp_download_rate + s.udp_download_rate) s.udp_download_rate
         )
     
     | CoreProtocol v -> 
         
-        Gui_com.gui_protocol_used := min v GuiEncoding.best_gui_version;
-        lprintf "Using protocol %d for communications" !Gui_com.gui_protocol_used;
-        lprint_newline ();
+        let version = min v GuiEncoding.best_gui_version in
+        for i = 0 to to_gui_last_opcode do
+          Gui_com.to_gui_protocol_used.(i) <- version;
+        done;
+        for i = 0 to from_gui_last_opcode do
+          Gui_com.from_gui_protocol_used.(i) <- version;
+        done;
+        lprintf "Using protocol %d for communications\n" version;
         gui#label_connect_status#set_text (gettext M.connected);
         Com.send (Password (!!O.login, !!O.password))
     
-    | Search_result (num,r) -> 
+    | Search_result (num,r,_) -> 
         begin try
             let r = Hashtbl.find G.results r in
             gui#tab_queries#h_search_result num r
           with _ -> 
-              lprintf "Exception in Search_result %d %d" num r;
-              lprint_newline ();
+              lprintf "Exception in Search_result %d %d\n" num r;
         end
     
     | Search_waiting (num,waiting) -> 
@@ -352,36 +334,45 @@ let value_reader gui t =
     
     | Options_info list ->
 (*        lprintf "Options_info"; lprint_newline ();*)
+        let module M = Options in
         let rec iter list =
           match list with
             [] -> ()
-          | (name, value) :: tail ->
+          | o :: tail ->
               (
                 try
                   let reference = 
-                    List.assoc name Gui_options.client_options_assocs 
+                    List.assoc o.M.option_name Gui_options.client_options_assocs 
                   in                  
-                  reference := value;
-                  Gui_config.add_option_value name reference
+                  reference := o.M.option_value;
+                  Gui_config.add_option_value o.M.option_name reference
                 with _ -> 
-                    Gui_config.add_option_value name (ref value)
+                    Gui_config.add_option_value o.M.option_name (ref o.M.option_value)
               );
               iter tail
         in
         iter list
     
-    | Add_section_option (section, message, option, optype) ->
-        let line = message, optype, option in
+    | Add_section_option (section, o) ->
+        let optype = match o.option_type with
+            "Bool" -> BoolEntry
+          | "Filename" -> FileEntry
+          | _ -> StringEntry in
+        let line = o.option_desc, optype, o.option_name in
         (try
             let options = List.assoc section !client_sections in
             if not (List.mem line !options) then
               options := !options @ [line]
-        with _ ->
-            client_sections := !client_sections  @[section, ref [line]]
+          with _ ->
+              client_sections := !client_sections  @[section, ref [line]]
         )          
     
-    | Add_plugin_option (section, message, option, optype) ->
-        let line = message, optype, option in
+    | Add_plugin_option (section, o) ->
+        let optype = match o.option_type with
+            "Bool" -> BoolEntry
+          | "Filename" -> FileEntry
+          | _ -> StringEntry in
+        let line = o.option_desc, optype, o.option_name in
         (try
             let options = List.assoc section !plugins_sections in
             if not (List.mem line !options) then
@@ -521,9 +512,15 @@ fichier selectionne. Si ca marche toujours dans ton interface, pas de
         GToolbox.message_box ~title: "Bad Password" 
         "Authorization Failed\nPlease, open the File->Settings menu and
           enter a valid password"
+    | GiftServerAttach _
+    | GiftServerStats _ -> assert false
+    | Uploaders l -> ()
+    | Pending l -> ()
+
+    | Search s -> ()
+
   with e ->
-      lprintf "Exception %s in reader" (Printexc2.to_string e);
-      lprint_newline ()
+      lprintf "Exception %s in reader\n" (Printexc2.to_string e)
 
       
 let generate_connect_menu gui =
@@ -548,11 +545,19 @@ let main () =
   let gui = new Gui_window.window () in
   let w = gui#window in
   let quit () = 
-    chmod_config (); 
     CommonGlobals.exit_properly 0
   in
   Gui_config.update_toolbars_style gui;
-  chmod_config (); 
+  Gui_global.top_menus := ("IM", (fun menu ->
+        
+        let menu_item =
+          GMenu.menu_item ~label: "IM Window"
+            ~packing:menu#add ()
+        in
+        ignore (menu_item#connect#activate ~callback:(fun _ ->              
+              Gui_im.main_window#window#show ()));
+
+    )) :: !Gui_global.top_menus;
   List.iter (fun (menu, init) ->
       let _Menu = GMenu.menu_item ~label:menu  ~packing:(gui#menubar#add) ()
       in
@@ -636,7 +641,6 @@ let main () =
         never_connected := false
   )
   
- 
 let _ = 
   CommonGlobals.gui_included := true;
   main ()
