@@ -34,6 +34,7 @@ open CommonComplexOptions
 open CommonFile
 open CommonDownloads
 open CommonInteractive
+open Autoconf
   
 open BTTypes
 open BTOptions
@@ -110,8 +111,8 @@ let _ =
         P.file_md4 = Md4.null;
         P.file_size = file_size file;
         P.file_downloaded = file_downloaded file;
-        P.file_nlocations = 0;
-        P.file_nclients = 0;
+        P.file_all_sources = 0;
+        P.file_active_sources = 0;
         P.file_state = file_state file;
         P.file_sources = None;
         P.file_download_rate = file_download_rate file.file_file;
@@ -338,12 +339,21 @@ let _ = (
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
 	onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
         
-        html_mods_td buf [
+        let show_emulemods_column = ref false in
+           if Autoconf.donkey = "yes" then begin
+               if !!emule_mods_count then
+                   show_emulemods_column := true
+        end;
+
+        html_mods_td buf ([
           ("", "srb ar", Printf.sprintf "%d" (client_num c));
           ((string_of_connection_state (client_state cc)), "sr", 
             (short_string_of_connection_state (client_state cc)));
           ((Sha1.to_string c.client_uid), "sr", cinfo.GuiTypes.client_name);
           ("", "sr", "bT"); (* cinfo.GuiTypes.client_software *)
+          ] @
+          (if !show_emulemods_column then [("", "sr", "")] else [])
+          @ [
           ("", "sr", "F");
           ("", "sr ar", Printf.sprintf "%d" 
               (((last_time ()) - cinfo.GuiTypes.client_connect_time) / 60));
@@ -351,7 +361,7 @@ let _ = (
           ("", "sr", (Ip.to_string (fst c.client_host)));
           ("", "sr ar", (size_of_int64 c.client_uploaded));
           ("", "sr ar", (size_of_int64 c.client_downloaded));
-          ("", "sr", info.GuiTypes.file_name); ];
+          ("", "sr", info.GuiTypes.file_name); ]);
         true
     )
 )
